@@ -109,12 +109,13 @@ def _mock_nsxlib():
     return mocking
 
 
-def get_default_nsxlib_config():
+def get_default_nsxlib_config(allow_passthrough=True):
     return config.NsxLibConfig(
         username=NSX_USER,
         password=NSX_PASSWORD,
         retries=NSX_HTTP_RETRIES,
         insecure=NSX_INSECURE,
+        token_provider=None,
         ca_file=NSX_CERT,
         concurrent_connections=NSX_CONCURENT_CONN,
         http_timeout=NSX_HTTP_TIMEOUT,
@@ -126,7 +127,8 @@ def get_default_nsxlib_config():
         plugin_tag=PLUGIN_TAG,
         plugin_ver=PLUGIN_VER,
         dns_nameservers=DNS_NAMESERVERS,
-        dns_domain=DNS_DOMAIN
+        dns_domain=DNS_DOMAIN,
+        allow_passthrough=allow_passthrough
     )
 
 
@@ -136,6 +138,7 @@ def get_nsxlib_config_with_client_cert():
         retries=NSX_HTTP_RETRIES,
         insecure=NSX_INSECURE,
         ca_file=NSX_CERT,
+        token_provider=None,
         concurrent_connections=NSX_CONCURENT_CONN,
         http_timeout=NSX_HTTP_TIMEOUT,
         http_read_timeout=NSX_HTTP_READ_TIMEOUT,
@@ -196,6 +199,9 @@ class MemoryMockAPIProvider(nsx_cluster.AbstractHTTPProvider):
     def is_connection_exception(self, exception):
         return isinstance(exception, requests_exceptions.ConnectionError)
 
+    def is_timeout_exception(self, exception):
+        return isinstance(exception, requests_exceptions.Timeout)
+
 
 class NsxClientTestCase(NsxLibTestCase):
 
@@ -212,19 +218,22 @@ class NsxClientTestCase(NsxLibTestCase):
             http_timeout=None,
             http_read_timeout=None,
             conn_idle_timeout=None,
-            nsx_api_managers=None):
+            nsx_api_managers=None,
+            max_attempts=None):
 
             nsxlib_config = config.NsxLibConfig(
                 username=username or NSX_USER,
                 password=password or NSX_PASSWORD,
                 retries=retries or NSX_HTTP_RETRIES,
                 insecure=insecure if insecure is not None else NSX_INSECURE,
+                token_provider=None,
                 ca_file=ca_file or NSX_CERT,
                 concurrent_connections=(concurrent_connections or
                                         NSX_CONCURENT_CONN),
                 http_timeout=http_timeout or NSX_HTTP_TIMEOUT,
                 http_read_timeout=http_read_timeout or NSX_HTTP_READ_TIMEOUT,
                 conn_idle_timeout=conn_idle_timeout or NSX_CONN_IDLE_TIME,
+                max_attempts=max_attempts or NSX_MAX_ATTEMPTS,
                 http_provider=NsxClientTestCase.MockHTTPProvider(
                     session_response=session_response),
                 nsx_api_managers=nsx_api_managers or [NSX_MANAGER],
