@@ -39,6 +39,13 @@ JSON_DFT_ACCEPT_HEADERS = {
     'Cookie': 'JSESSIONID=%s;' % nsxlib_testcase.JSESSIONID
 }
 
+PARTIAL_UPDATE_HEADERS = {
+    'Accept': 'application/json',
+    'Content-Type': 'application/json',
+    'Cookie': 'JSESSIONID=%s;' % nsxlib_testcase.JSESSIONID,
+    'nsx-enable-partial-patch': 'true'
+}
+
 
 def _headers(**kwargs):
     headers = copy.copy(DFT_ACCEPT_HEADERS)
@@ -249,7 +256,10 @@ class NsxV3RESTClientTestCase(nsxlib_testcase.NsxClientTestCase):
                 content = jsonutils.dumps({'httpStatus': 'dummy',
                                            'error_code': error_code,
                                            'module_name': 'dummy',
-                                           'error_message': 'bad'})
+                                           'error_message': 'bad',
+                                           'related_errors': [{
+                                               'error_message': 'bla',
+                                               'error_code': 'code'}]})
             response = mocks.MockRequestsResponse(
                 status_code, content)
 
@@ -263,18 +273,16 @@ class NsxV3RESTClientTestCase(nsxlib_testcase.NsxClientTestCase):
         for verb in ['get', 'post', 'put', 'delete']:
             for code in client.RESTClient._VERB_RESP_CODES.get(verb):
                 _verb_response_code(verb, code)
-            self.assertRaises(
-                nsxlib_exc.ManagerError,
-                _verb_response_code, verb,
-                requests.codes.INTERNAL_SERVER_ERROR)
-            self.assertRaises(
-                nsxlib_exc.ResourceNotFound,
-                _verb_response_code, verb,
-                requests.codes.NOT_FOUND)
-            self.assertRaises(
-                nsxlib_exc.BackendResourceNotFound,
-                _verb_response_code, verb,
-                requests.codes.NOT_FOUND, 202)
+            with self.assertRaises(nsxlib_exc.ManagerError) as e:
+                _verb_response_code(verb, requests.codes.INTERNAL_SERVER_ERROR)
+            self.assertEqual(e.exception.status_code,
+                             requests.codes.INTERNAL_SERVER_ERROR)
+            with self.assertRaises(nsxlib_exc.ResourceNotFound) as e:
+                _verb_response_code(verb, requests.codes.NOT_FOUND)
+            self.assertEqual(e.exception.status_code, requests.codes.NOT_FOUND)
+            with self.assertRaises(nsxlib_exc.BackendResourceNotFound) as e:
+                _verb_response_code(verb, requests.codes.NOT_FOUND, 202)
+            self.assertEqual(e.exception.status_code, requests.codes.NOT_FOUND)
 
     def test_inject_headers_callback(self):
 
@@ -302,6 +310,10 @@ class NsxV3RESTClientTestCase(nsxlib_testcase.NsxClientTestCase):
         utils.set_inject_headers_callback(None)
         self.assertIsNotNone(self.injected)
 
+    def test_http_error_to_exception(self):
+        exc = client.http_error_to_exception(500, 607)
+        self.assertEqual(exc, nsxlib_exc.APITransactionAborted)
+
 
 class NsxV3JSONClientTestCase(nsxlib_testcase.NsxClientTestCase):
 
@@ -323,12 +335,11 @@ class NsxV3JSONClientTestCase(nsxlib_testcase.NsxClientTestCase):
         self.assertEqual(resp, {'result': {'ok': 200}})
 
     def test_mask_password(self):
-        pwds = ('my!pwd0#', 'some0therlong$pwd', 'pwd')
+        pwds = ('my!pwd0#', 'some0therlong$pwd')
         body = {'name_pwd': 'name1',
                 'password': pwds[0],
                 'some_list': {'name_password': 'name2',
-                              'password': pwds[1]},
-                'password': pwds[2]}
+                              'password': pwds[1]}}
 
         cl = client.RESTClient(None)
         json_body = jsonutils.dumps(body)
@@ -349,6 +360,13 @@ class NsxV3APIClientTestCase(nsxlib_testcase.NsxClientTestCase):
             'get', api,
             'https://1.2.3.4/api/v1/ports')
 
+    def test_raise_error(self):
+        api = self.new_mocked_client(client.NSX3Client)
+        with self.assertRaises(nsxlib_exc.ManagerError) as e:
+            api._raise_error(requests.codes.INTERNAL_SERVER_ERROR, 'GET', '')
+        self.assertEqual(e.exception.status_code,
+                         requests.codes.INTERNAL_SERVER_ERROR)
+
 
 # NOTE(boden): remove this when tmp brigding removed
 class NsxV3APIClientBridgeTestCase(nsxlib_testcase.NsxClientTestCase):
