@@ -32,17 +32,6 @@ monitor_types = load_balancer.MonitorTypes
 monitors = [monitor_types.HTTP, monitor_types.HTTPS, monitor_types.ICMP,
             monitor_types.PASSIVE, monitor_types.TCP, monitor_types.UDP]
 
-tags = [
-    {
-        'scope': 'os-project-id',
-        'tag': 'project-1'
-    },
-    {
-        'scope': 'os-api-version',
-        'tag': '2.1.1.0'
-    }
-]
-
 
 class TestApplicationProfile(nsxlib_testcase.NsxClientTestCase):
 
@@ -53,17 +42,96 @@ class TestApplicationProfile(nsxlib_testcase.NsxClientTestCase):
                 'display_name': fake_profile['display_name'],
                 'description': fake_profile['description'],
                 'resource_type': profile_type,
-                'tags': tags
+                'tags': consts.FAKE_TAGS
             }
             with mock.patch.object(self.nsxlib.client, 'create') as create:
                 self.nsxlib.load_balancer.application_profile.create(
                     display_name=body['display_name'],
                     description=body['description'],
                     resource_type=body['resource_type'],
-                    tags=tags)
+                    tags=consts.FAKE_TAGS)
                 create.assert_called_with('loadbalancer/application-profiles',
                                           body)
 
+    def test_create_fast_tcp_profiles(self):
+        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
+        body = {
+            'display_name': fake_profile['display_name'],
+            'description': fake_profile['description'],
+            'resource_type': app_profile_types.FAST_TCP,
+            'close_timeout': 8,
+            'ha_flow_mirroring_enabled': True,
+            'idle_timeout': 1800,
+            'tags': consts.FAKE_TAGS
+        }
+        with mock.patch.object(self.nsxlib.client, 'create') as create:
+            self.nsxlib.load_balancer.application_profile.create(
+                display_name=body['display_name'],
+                description=body['description'],
+                resource_type=body['resource_type'],
+                close_timeout=body['close_timeout'],
+                ha_flow_mirroring_enabled=body['ha_flow_mirroring_enabled'],
+                idle_timeout=body['idle_timeout'],
+                tags=consts.FAKE_TAGS)
+            create.assert_called_with('loadbalancer/application-profiles',
+                                      body)
+
+    def test_create_fast_udp_profiles(self):
+        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
+        body = {
+            'display_name': fake_profile['display_name'],
+            'description': fake_profile['description'],
+            'resource_type': app_profile_types.FAST_UDP,
+            'flow_mirroring_enabled': True,
+            'idle_timeout': 1800,
+            'tags': consts.FAKE_TAGS
+        }
+        with mock.patch.object(self.nsxlib.client, 'create') as create:
+            self.nsxlib.load_balancer.application_profile.create(
+                display_name=body['display_name'],
+                description=body['description'],
+                resource_type=body['resource_type'],
+                flow_mirroring_enabled=body['flow_mirroring_enabled'],
+                idle_timeout=body['idle_timeout'],
+                tags=consts.FAKE_TAGS)
+            create.assert_called_with('loadbalancer/application-profiles',
+                                      body)
+
+    def test_create_http_profiles(self):
+        fake_profile = consts.FAKE_APPLICATION_PROFILE.copy()
+        body = {
+            'display_name': fake_profile['display_name'],
+            'description': fake_profile['description'],
+            'resource_type': app_profile_types.HTTP,
+            'http_redirect_to': fake_profile['http_redirect_to'],
+            'http_redirect_to_https': fake_profile['http_redirect_to_https'],
+            'ntlm': fake_profile['ntlm'],
+            'request_body_size': fake_profile['request_body_size'],
+            'request_header_size': fake_profile['request_header_size'],
+            'response_header_size': fake_profile['response_header_size'],
+            'response_timeout': fake_profile['response_timeout'],
+            'x_forwarded_for': fake_profile['x_forwarded_for'],
+            'idle_timeout': fake_profile['idle_timeout'],
+            'tags': consts.FAKE_TAGS
+        }
+        with mock.patch.object(self.nsxlib.client, 'create') as create:
+            self.nsxlib.load_balancer.application_profile.create(
+                display_name=body['display_name'],
+                description=body['description'],
+                resource_type=body['resource_type'],
+                http_redirect_to=body['http_redirect_to'],
+                http_redirect_to_https=body['http_redirect_to_https'],
+                ntlm=body['ntlm'],
+                request_body_size=body['request_body_size'],
+                request_header_size=body['request_header_size'],
+                response_header_size=body['response_header_size'],
+                response_timeout=body['response_timeout'],
+                x_forwarded_for=body['x_forwarded_for'],
+                idle_timeout=body['idle_timeout'],
+                tags=consts.FAKE_TAGS)
+            create.assert_called_with('loadbalancer/application-profiles',
+                                      body)
+
     def test_list_application_profiles(self):
         with mock.patch.object(self.nsxlib.client, 'list') as list_call:
             self.nsxlib.load_balancer.application_profile.list()
@@ -96,12 +164,12 @@ class TestPersistenceProfile(nsxlib_testcase.NsxClientTestCase):
                 'display_name': fake_profile['display_name'],
                 'description': fake_profile['description'],
                 'resource_type': profile_type,
-                'tags': tags
+                'tags': consts.FAKE_TAGS
             }
             with mock.patch.object(self.nsxlib.client, 'create') as create:
                 self.nsxlib.load_balancer.persistence_profile.create(
-                    body['display_name'], body['description'], tags,
-                    body['resource_type'])
+                    body['display_name'], body['description'],
+                    consts.FAKE_TAGS, body['resource_type'])
                 create.assert_called_with('loadbalancer/persistence-profiles',
                                           body)
 
@@ -137,7 +205,7 @@ class TestRule(nsxlib_testcase.NsxClientTestCase):
             'resource_type': fake_rule['resource_type'],
             'phase': fake_rule['phase'],
             'match_strategy': fake_rule['match_strategy'],
-            'tags': tags
+            'tags': consts.FAKE_TAGS
         }
         with mock.patch.object(self.nsxlib.client, 'create') as create:
             self.nsxlib.load_balancer.rule.create(**body)
@@ -169,13 +237,13 @@ class TestClientSslProfile(nsxlib_testcase.NsxClientTestCase):
         body = {
             'display_name': fake_profile['display_name'],
             'description': fake_profile['description'],
-            'tags': tags
+            'tags': consts.FAKE_TAGS
         }
         with mock.patch.object(self.nsxlib.client, 'create') as create:
-                self.nsxlib.load_balancer.client_ssl_profile.create(
-                    body['display_name'], body['description'], tags)
-                create.assert_called_with('loadbalancer/client-ssl-profiles',
-                                          body)
+            self.nsxlib.load_balancer.client_ssl_profile.create(
+                body['display_name'], body['description'],
+                consts.FAKE_TAGS)
+            create.assert_called_with('loadbalancer/client-ssl-profiles', body)
 
     def test_list_client_ssl_profiles(self):
         with mock.patch.object(self.nsxlib.client, 'list') as list_call:
@@ -207,13 +275,13 @@ class TestServerSslProfile(nsxlib_testcase.NsxClientTestCase):
         body = {
             'display_name': fake_profile['display_name'],
             'description': fake_profile['description'],
-            'tags': tags
+            'tags': consts.FAKE_TAGS
         }
         with mock.patch.object(self.nsxlib.client, 'create') as create:
-                self.nsxlib.load_balancer.server_ssl_profile.create(
-                    body['display_name'], body['description'], tags)
-                create.assert_called_with('loadbalancer/server-ssl-profiles',
-                                          body)
+            self.nsxlib.load_balancer.server_ssl_profile.create(
+                body['display_name'], body['description'],
+                consts.FAKE_TAGS)
+            create.assert_called_with('loadbalancer/server-ssl-profiles', body)
 
     def test_list_server_ssl_profiles(self):
         with mock.patch.object(self.nsxlib.client, 'list') as list_call:
@@ -247,12 +315,12 @@ class TestMonitor(nsxlib_testcase.NsxClientTestCase):
                 'display_name': fake_monitor['display_name'],
                 'description': fake_monitor['description'],
                 'resource_type': monitor_type,
-                'tags': tags
+                'tags': consts.FAKE_TAGS
             }
             with mock.patch.object(self.nsxlib.client, 'create') as create:
                 self.nsxlib.load_balancer.monitor.create(
-                    body['display_name'], body['description'], tags,
-                    body['resource_type'])
+                    body['display_name'], body['description'],
+                    consts.FAKE_TAGS, body['resource_type'])
                 create.assert_called_with('loadbalancer/monitors',
                                           body)
 
@@ -284,14 +352,14 @@ class TestPool(nsxlib_testcase.NsxClientTestCase):
             'display_name': fake_pool['display_name'],
             'description': fake_pool['description'],
             'algorithm': fake_pool['algorithm'],
-            'tags': tags
+            'tags': consts.FAKE_TAGS
         }
         with mock.patch.object(self.nsxlib.client, 'create') as create:
-                self.nsxlib.load_balancer.pool.create(
-                    body['display_name'], body['description'], tags,
-                    algorithm=body['algorithm'])
-                create.assert_called_with('loadbalancer/pools',
-                                          body)
+            self.nsxlib.load_balancer.pool.create(
+                body['display_name'], body['description'],
+                consts.FAKE_TAGS, algorithm=body['algorithm'])
+            create.assert_called_with('loadbalancer/pools',
+                                      body)
 
     def test_list_pools(self):
         with mock.patch.object(self.nsxlib.client, 'list') as list_call:
@@ -365,15 +433,15 @@ class TestVirtualServer(nsxlib_testcase.NsxClientTestCase):
             'ip_protocol': fake_virtual_server['ip_protocol'],
             'port': fake_virtual_server['port'],
             'enabled': fake_virtual_server['enabled'],
-            'tags': tags
+            'tags': consts.FAKE_TAGS
         }
         with mock.patch.object(self.nsxlib.client, 'create') as create:
-                self.nsxlib.load_balancer.virtual_server.create(
-                    body['display_name'], body['description'], tags,
-                    ip_protocol=body['ip_protocol'], port=body['port'],
-                    enabled=body['enabled'])
-                create.assert_called_with('loadbalancer/virtual-servers',
-                                          body)
+            self.nsxlib.load_balancer.virtual_server.create(
+                body['display_name'], body['description'],
+                consts.FAKE_TAGS, ip_protocol=body['ip_protocol'],
+                port=body['port'], enabled=body['enabled'])
+            create.assert_called_with('loadbalancer/virtual-servers',
+                                      body)
 
     def test_list_virtual_servers(self):
         with mock.patch.object(self.nsxlib.client, 'list') as list_call:
@@ -509,14 +577,19 @@ class TestService(nsxlib_testcase.NsxClientTestCase):
             'description': fake_service['description'],
             'enabled': fake_service['enabled'],
             'attachment': fake_service['attachment'],
-            'tags': tags
+            'relax_scale_validation': fake_service['relax_scale_validation'],
+            'tags': consts.FAKE_TAGS
         }
-        with mock.patch.object(self.nsxlib.client, 'create') as create:
-                self.nsxlib.load_balancer.service.create(
-                    body['display_name'], body['description'], tags,
-                    enabled=body['enabled'], attachment=body['attachment'])
-                create.assert_called_with('loadbalancer/services',
-                                          body)
+        with mock.patch.object(self.nsxlib.client, 'create') as create, \
+                mock.patch.object(self.nsxlib, 'feature_supported') as support:
+            support.return_value = True
+            self.nsxlib.load_balancer.service.create(
+                body['display_name'], body['description'],
+                consts.FAKE_TAGS, enabled=body['enabled'],
+                attachment=body['attachment'],
+                relax_scale_validation=body['relax_scale_validation'])
+            create.assert_called_with('loadbalancer/services',
+                                      body)
 
     def test_list_services(self):
         with mock.patch.object(self.nsxlib.client, 'list') as list_call:
