@@ -36,11 +36,19 @@ def http_error_to_exception(status_code, error_code):
              'default': exceptions.ResourceNotFound},
         requests.codes.BAD_REQUEST:
             {'60508': exceptions.NsxIndexingInProgress,
-             '500045': exceptions.NsxPendingDelete},
+             '60514': exceptions.NsxSearchTimeout,
+             '60515': exceptions.NsxSearchOutOfSync,
+             '8327': exceptions.NsxOverlapVlan,
+             '500045': exceptions.NsxPendingDelete,
+             '500030': exceptions.ResourceInUse,
+             '500105': exceptions.NsxOverlapAddresses,
+             '503040': exceptions.NsxSegemntWithVM},
         requests.codes.CONFLICT: exceptions.StaleRevision,
         requests.codes.PRECONDITION_FAILED: exceptions.StaleRevision,
         requests.codes.INTERNAL_SERVER_ERROR:
-            {'99': exceptions.ClientCertificateNotTrusted},
+            {'98': exceptions.CannotConnectToServer,
+             '99': exceptions.ClientCertificateNotTrusted,
+             '607': exceptions.APITransactionAborted},
         requests.codes.FORBIDDEN:
             {'98': exceptions.BadXSRFToken},
         requests.codes.TOO_MANY_REQUESTS: exceptions.TooManyRequests,
@@ -89,8 +97,9 @@ class RESTClient(object):
     def list(self, resource='', headers=None, silent=False):
         return self.url_list(resource, headers=headers, silent=silent)
 
-    def get(self, uuid, headers=None, silent=False):
-        return self.url_get(uuid, headers=headers, silent=silent)
+    def get(self, uuid, headers=None, silent=False, with_retries=True):
+        return self.url_get(uuid, headers=headers, silent=silent,
+                            with_retries=with_retries)
 
     def delete(self, uuid, headers=None, expected_results=None):
         return self.url_delete(uuid, headers=headers,
@@ -109,7 +118,8 @@ class RESTClient(object):
         return self.url_patch(resource, body, headers=headers)
 
     def url_list(self, url, headers=None, silent=False):
-        concatenate_response = self.url_get(url, headers=headers)
+        concatenate_response = self.url_get(url, headers=headers,
+                                            silent=silent)
         cursor = concatenate_response.get('cursor', NULL_CURSOR_PREFIX)
         op = '&' if urlparse.urlparse(url).query else '?'
         url += op + 'cursor='
@@ -120,9 +130,9 @@ class RESTClient(object):
             cursor = page.get('cursor', NULL_CURSOR_PREFIX)
         return concatenate_response
 
-    def url_get(self, url, headers=None, silent=False):
+    def url_get(self, url, headers=None, silent=False, with_retries=True):
         return self._rest_call(url, method='GET', headers=headers,
-                               silent=silent)
+                               silent=silent, with_retries=with_retries)
 
     def url_delete(self, url, headers=None, expected_results=None):
         return self._rest_call(url, method='DELETE', headers=headers,
@@ -140,10 +150,12 @@ class RESTClient(object):
         return self._rest_call(url, method='PATCH', body=body, headers=headers)
 
     def _raise_error(self, status_code, operation, result_msg,
-                     error_code=None):
+                     error_code=None, related_error_codes=None):
         error = http_error_to_exception(status_code, error_code)
         raise error(manager='', operation=operation, details=result_msg,
-                    error_code=error_code)
+                    error_code=error_code,
+                    related_error_codes=related_error_codes,
+                    status_code=status_code)
 
     def _validate_result(self, result, expected, operation, silent=False):
         if result.status_code not in expected:
@@ -158,16 +170,21 @@ class RESTClient(object):
                              'body': result_msg})
 
             error_code = None
+            related_error_codes = []
             if isinstance(result_msg, dict) and 'error_message' in result_msg:
                 error_code = result_msg.get('error_code')
                 related_errors = [error['error_message'] for error in
                                   result_msg.get('related_errors', [])]
+                related_error_codes = [str(error['error_code']) for error in
+                                       result_msg.get('related_errors', []) if
+                                       error.get('error_code')]
                 result_msg = result_msg['error_message']
                 if related_errors:
                     result_msg += " relatedErrors: %s" % ' '.join(
                         related_errors)
             self._raise_error(result.status_code, operation, result_msg,
-                              error_code=error_code)
+                              error_code=error_code,
+                              related_error_codes=related_error_codes)
 
     @classmethod
     def merge_headers(cls, *headers):
@@ -195,7 +212,7 @@ class RESTClient(object):
         return re.sub(pattern, '"password": "********"', json)
 
     def _rest_call(self, url, method='GET', body=None, headers=None,
-                   silent=False, expected_results=None):
+                   silent=False, expected_results=None, **kwargs):
         request_headers = headers.copy() if headers else {}
         request_headers.update(self._default_headers)
         if utils.INJECT_HEADERS_CALLBACK:
@@ -208,7 +225,8 @@ class RESTClient(object):
         if not silent:
 
             LOG.debug("REST call: %s %s. Headers: %s. Body: %s",
-                      method, request_url, request_headers,
+                      method, request_url,
+                      utils.censor_headers(request_headers),
                       self._mask_password(body))
 
         ts = time.time()
@@ -296,24 +314,32 @@ class NSX3Client(JSONRESTClient):
             client_obj=client_obj)
 
     def _raise_error(self, status_code, operation, result_msg,
-                     error_code=None):
+                     error_code=None, related_error_codes=None):
         """Override the Rest client errors to add the manager IPs"""
         error = http_error_to_exception(status_code, error_code)
         raise error(manager=self.nsx_api_managers,
                     operation=operation,
                     details=result_msg,
-                    error_code=error_code)
+                    error_code=error_code,
+                    related_error_codes=related_error_codes,
+                    status_code=status_code)
 
     def _rest_call(self, url, **kwargs):
-        if self.rate_limit_retry:
-            # If too many requests are handled by the nsx at the same time,
-            # error "429: Too Many Requests" or "503: Server Unavailable"
-            # will be returned.
+        if kwargs.get('with_retries', True):
+            # Retry on "607: Persistence layer is currently reconfiguring"
+            # and on "98: Cannot connect to server"
+            retry_codes = [exceptions.APITransactionAborted,
+                           exceptions.CannotConnectToServer]
+            if self.rate_limit_retry:
+                # If too many requests are handled by the nsx at the same time,
+                # error "429: Too Many Requests" or "503: Server Unavailable"
+                # will be returned.
+                retry_codes.append(exceptions.ServerBusy)
+
             # the client is expected to retry after a random 400-600 milli,
             # and later exponentially until 5 seconds wait
             @utils.retry_random_upon_exception(
-                exceptions.ServerBusy,
-                max_attempts=self.max_attempts)
+                tuple(retry_codes), max_attempts=self.max_attempts)
             def _rest_call_with_retry(self, url, **kwargs):
                 return super(NSX3Client, self)._rest_call(url, **kwargs)
             return _rest_call_with_retry(self, url, **kwargs)
