@@ -34,6 +34,7 @@ from requests import exceptions as requests_exceptions
 import six
 import six.moves.urllib.parse as urlparse
 import tenacity
+import urllib3
 
 from vmware_nsxlib._i18n import _
 from vmware_nsxlib.v3 import client as nsx_client
@@ -45,7 +46,10 @@ LOG = log.getLogger(__name__)
 
 # disable warning message for each HTTP retry
 logging.getLogger(
-    "requests.packages.urllib3.connectionpool").setLevel(logging.ERROR)
+    "urllib3.connectionpool").setLevel(logging.ERROR)
+
+# Hide the InsecureRequestWarning from urllib3
+urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -86,6 +90,13 @@ class AbstractHTTPProvider(object):
         Return True if it's a connection exception and False otherwise.
         """
 
+    @abc.abstractmethod
+    def is_timeout_exception(self, exception):
+        """Determine if the given exception is related to timeout.
+
+        Return True if it's a timeout exception and False otherwise.
+        """
+
 
 class TimeoutSession(requests.Session):
     """Extends requests.Session to support timeout at the session level."""
@@ -179,9 +190,15 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
             conn, url_prefix=endpoint.provider.url,
             url_path_base=cluster_api.nsxlib_config.url_base,
             default_headers=conn.default_headers)
+
+        # Check the manager state directly
+        if cluster_api.nsxlib_config.validate_connection_method:
+            cluster_api.nsxlib_config.validate_connection_method(
+                client, endpoint.provider.url)
+
+        # If keeplive section returns a list, it is assumed to be non-empty
         keepalive_section = cluster_api.nsxlib_config.keepalive_section
         result = client.get(keepalive_section, silent=True)
-        # If keeplive section returns a list, it is assumed to be non-empty
         if not result or result.get('result_count', 1) <= 0:
             msg = _("No %(section)s found "
                     "for '%(url)s'") % {'section': keepalive_section,
@@ -196,76 +213,130 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
                                  config.http_read_timeout)
         if config.client_cert_provider:
             session.cert_provider = config.client_cert_provider
-        else:
+        # Set the headers with Auth info when token provider is set,
+        # otherwise set the username and password
+        elif not config.token_provider:
             session.auth = (provider.username, provider.password)
 
         # NSX v3 doesn't use redirects
         session.max_redirects = 0
 
-        session.verify = not config.insecure
-        if session.verify and provider.ca_file:
+        if config.insecure:
+            # no verification on server certificate
+            session.verify = False
+            thumbprint = None
+        elif provider.ca_file:
             # verify using the said ca bundle path
             session.verify = provider.ca_file
+            thumbprint = None
+        elif provider.thumbprint:
+            # verify using the thumbprint
+            session.verify = None
+            thumbprint = provider.thumbprint
+        else:
+            # verify using the default system root CAs
+            session.verify = True
+            thumbprint = None
 
         # we are pooling with eventlet in the cluster class
-        adapter = adapters.HTTPAdapter(
+        adapter = NSXHTTPAdapter(
             pool_connections=1, pool_maxsize=1,
             max_retries=config.retries,
-            pool_block=False)
+            pool_block=False, thumbprint=thumbprint)
         session.mount('http://', adapter)
         session.mount('https://', adapter)
 
         self.get_default_headers(session, provider,
-                                 config.allow_overwrite_header)
+                                 config.allow_overwrite_header,
+                                 config.token_provider)
 
         return session
 
     def is_connection_exception(self, exception):
         return isinstance(exception, requests_exceptions.ConnectionError)
 
+    def is_timeout_exception(self, exception):
+        return isinstance(exception, requests_exceptions.Timeout)
+
     def is_conn_open_exception(self, exception):
         return isinstance(exception, requests_exceptions.ConnectTimeout)
 
-    def get_default_headers(self, session, provider, allow_overwrite_header):
+    def get_default_headers(self, session, provider, allow_overwrite_header,
+                            token_provider=None):
         """Get the default headers that should be added to future requests"""
         session.default_headers = {}
 
+        # Add allow-overwrite if configured
+        if allow_overwrite_header:
+            session.default_headers['X-Allow-Overwrite'] = 'true'
         # Perform the initial session create and get the relevant jsessionid &
         # X-XSRF-TOKEN for future requests
         req_data = ''
-        if not session.cert_provider:
+        req_headers = {'Accept': 'application/json',
+                       'Content-Type': 'application/x-www-form-urlencoded'}
+        # Insert the JWT in Auth header if using tokens for auth
+        if token_provider:
+            try:
+                token_value = token_provider.get_token()
+                bearer_token = token_provider.get_header_value(token_value)
+                token_header = {"Authorization": bearer_token}
+                session.default_headers.update(token_header)
+                req_headers.update(token_header)
+            except exceptions.BadJSONWebTokenProviderRequest as e:
+                LOG.error("Session create failed for endpoint %s due to "
+                          "error in retrieving JSON Web Token: %s",
+                          provider.url, e)
+        elif not session.cert_provider:
             # With client certificate authentication, username and password
             # may not be provided.
             # If provided, backend treats these credentials as authentication
             # and ignores client cert as principal identity indication.
             req_data = 'j_username=%s&j_password=%s' % (provider.username,
                                                         provider.password)
-        req_headers = {'Accept': 'application/json',
-                       'Content-Type': 'application/x-www-form-urlencoded'}
         # Cannot use the certificate at this stage, because it is used for
         # the certificate generation
-        resp = session.request('post', provider.url + self.SESSION_CREATE_URL,
-                               data=req_data, headers=req_headers)
-        if resp.status_code != 200:
-            LOG.error("Session create failed for endpoint %s", provider.url)
-            # this will later cause the endpoint to be Down
+        try:
+            resp = session.request(
+                'post', provider.url + self.SESSION_CREATE_URL,
+                data=req_data, headers=req_headers)
+        except Exception as e:
+            # Error 403 might be because the backend does not support this for
+            # all versions.
+            LOG.warning("Session create failed for endpoint %s with error %s",
+                        provider.url, e)
         else:
-            for header_name in resp.headers:
-                if self.SET_COOKIE_FIELD.lower() == header_name.lower():
-                    m = re.match('%s=.*?\;' % self.JSESSIONID,
-                                 resp.headers[header_name])
-                    if m:
-                        session.default_headers[self.COOKIE_FIELD] = m.group()
-                if self.XSRF_TOKEN.lower() == header_name.lower():
-                    session.default_headers[self.XSRF_TOKEN] = resp.headers[
-                        header_name]
-            LOG.info("Session create succeeded for endpoint %(url)s with "
-                     "headers %(hdr)s",
-                     {'url': provider.url, 'hdr': session.default_headers})
+            if resp.status_code != 200 and resp.status_code != 201:
+                LOG.warning("Session create failed for endpoint %s with "
+                            "response %s",
+                            provider.url, resp.status_code)
+                # this may will later cause the endpoint to be Down
+            else:
+                for header_name in resp.headers:
+                    if self.SET_COOKIE_FIELD.lower() == header_name.lower():
+                        m = re.match('%s=.*?\;' % self.JSESSIONID,  # noqa
+                                     resp.headers[header_name])
+                        if m:
+                            session.default_headers[self.COOKIE_FIELD] = (
+                                m.group())
+                    if self.XSRF_TOKEN.lower() == header_name.lower():
+                        session.default_headers[self.XSRF_TOKEN] = (
+                            resp.headers[header_name])
+                LOG.info("Session create succeeded for endpoint %(url)s with "
+                         "headers %(hdr)s",
+                         {'url': provider.url,
+                          'hdr':
+                          utils.censor_headers(session.default_headers)})
 
-        # Add allow-overwrite if configured
-        if allow_overwrite_header:
-            session.default_headers['X-Allow-Overwrite'] = 'true'
+
+class NSXHTTPAdapter(adapters.HTTPAdapter):
+    def __init__(self, *args, **kwargs):
+        self.thumbprint = kwargs.pop("thumbprint", None)
+        super(NSXHTTPAdapter, self).__init__(*args, **kwargs)
+
+    def init_poolmanager(self, *args, **kwargs):
+        if self.thumbprint:
+            kwargs["assert_fingerprint"] = self.thumbprint
+        super(NSXHTTPAdapter, self).init_poolmanager(*args, **kwargs)
 
 
 class ClusterHealth(object):
@@ -297,12 +368,14 @@ class Provider(object):
     Which has a unique id a connection URL, and the credential details.
     """
 
-    def __init__(self, provider_id, provider_url, username, password, ca_file):
+    def __init__(self, provider_id, provider_url, username, password, ca_file,
+                 thumbprint=None):
         self.id = provider_id
         self.url = provider_url
         self.username = username
         self.password = password
         self.ca_file = ca_file
+        self.thumbprint = thumbprint
 
     def __str__(self):
         return str(self.url)
@@ -584,46 +657,51 @@ class ClusteredAPI(object):
         return _call_proxy
 
     def _proxy(self, proxy_for, uri, *args, **kwargs):
-        # proxy http request call to an avail endpoint
-        with self.endpoint_connection() as conn_data:
-            conn = conn_data.connection
-            endpoint = conn_data.endpoint
 
-            # http conn must support requests style interface
-            do_request = getattr(conn, proxy_for)
+        @utils.retry_upon_none_result(self.nsxlib_config.max_attempts)
+        def _proxy_internal(proxy_for, uri, *args, **kwargs):
+            # proxy http request call to an avail endpoint
+            with self.endpoint_connection() as conn_data:
+                conn = conn_data.connection
+                endpoint = conn_data.endpoint
 
-            if not uri.startswith('/'):
-                uri = "/%s" % uri
-            url = "%s%s" % (endpoint.provider.url, uri)
-            try:
-                LOG.debug("API cluster proxy %s %s to %s",
-                          proxy_for.upper(), uri, url)
-                # Add the connection default headers
-                if conn.default_headers:
-                    kwargs['headers'] = kwargs.get('headers', {})
-                    kwargs['headers'].update(conn.default_headers)
+                # http conn must support requests style interface
+                do_request = getattr(conn, proxy_for)
 
-                # call the actual connection method to do the
-                # http request/response over the wire
-                response = do_request(url, *args, **kwargs)
-                endpoint.set_state(EndpointState.UP)
+                if not uri.startswith('/'):
+                    uri = "/%s" % uri
+                url = "%s%s" % (endpoint.provider.url, uri)
+                try:
+                    LOG.debug("API cluster proxy %s %s to %s",
+                              proxy_for.upper(), uri, url)
+                    # Add the connection default headers
+                    if conn.default_headers:
+                        kwargs['headers'] = kwargs.get('headers', {})
+                        kwargs['headers'].update(conn.default_headers)
 
-                return response
-            except Exception as e:
-                LOG.warning("Request failed due to: %s", e)
-                if not self._http_provider.is_connection_exception(e):
-                    # only trap and retry connection errors
-                    raise e
-                if self._http_provider.is_conn_open_exception(e):
-                    # unable to establish new connection - endpoint is
-                    # inaccessible
-                    endpoint.set_state(EndpointState.DOWN)
+                    # call the actual connection method to do the
+                    # http request/response over the wire
+                    response = do_request(url, *args, **kwargs)
+                    endpoint.set_state(EndpointState.UP)
 
-                LOG.debug("Connection to %s failed, checking additional "
-                          "connections and endpoints" % url)
-                # this might be a result of server closing connection
-                # retry until exhausting connections and endpoints
-                return self._proxy(proxy_for, uri, *args, **kwargs)
+                    return response
+                except Exception as e:
+                    LOG.warning("Request failed due to: %s", e)
+                    if (not self._http_provider.is_connection_exception(e) and
+                        not self._http_provider.is_timeout_exception(e)):
+                        # only trap and retry connection & timeout errors
+                        raise e
+                    if self._http_provider.is_conn_open_exception(e):
+                        # unable to establish new connection - endpoint is
+                        # inaccessible
+                        endpoint.set_state(EndpointState.DOWN)
+
+                    LOG.info("Connection to %s failed, checking additional "
+                             "connections and endpoints" % url)
+                    # this might be a result of server closing connection
+                    # return None so it will retry upto max_attempts.
+
+        return _proxy_internal(proxy_for, uri, *args, **kwargs)
 
 
 class NSXClusteredAPI(ClusteredAPI):
@@ -670,5 +748,6 @@ class NSXClusteredAPI(ClusteredAPI):
                     urlparse.urlunparse(conf_url),
                     self.nsxlib_config.username(provider_index),
                     self.nsxlib_config.password(provider_index),
-                    self.nsxlib_config.ca_file(provider_index)))
+                    self.nsxlib_config.ca_file(provider_index),
+                    self.nsxlib_config.thumbprint(provider_index)))
         return providers
