@@ -35,13 +35,21 @@ class NsxLibConfig(object):
                              instead of basic authentication.
     :param insecure: If true, the NSX Manager server certificate is not
                      verified. If false the CA bundle specified via "ca_file"
-                     will be used or if unset the default system root CAs
+                     will be used or if unset the "thumbprint" will be used.
+                     If "thumbprint" is unset, the default system root CAs
                      will be used.
     :param ca_file: Specify a CA bundle file to use in verifying the NSX
                     Manager server certificate. This option is ignored if
-                    "insecure" is set to True. If "insecure" is set to
-                    False and ca_file is unset, the system root CAs will
-                    be used to verify the server certificate.
+                    "insecure" is set to True. If "insecure" is set to False
+                    and "ca_file" is unset, the "thumbprint" will be used.
+                    If "thumbprint" is unset, the system root CAs will be
+                    used to verify the server certificate.
+    :param thumbprint: Specify a thumbprint string to use in verifying the
+                       NSX Manager server certificate. This option is ignored
+                       if "insecure" is set to True or "ca_file" is defined.
+    :param token_provider: None, or instance of implemented AbstractJWTProvider
+                           which will return the JSON Web Token used in the
+                           requests in NSX for authorization.
 
     :param concurrent_connections: Maximum concurrent connections to each NSX
                                    manager.
@@ -78,6 +86,14 @@ class NsxLibConfig(object):
                                       endpoint in the NSX management cluster is
                                       available to serve a request, and retry
                                       the request instead.
+
+    -- Additional parameters which are relevant only for the Policy manager:
+    :param allow_passthrough: If True, use nsx manager api for cases which are
+                              not supported by the policy manager api.
+    :param realization_max_attempts: Maximum number of times to retry while
+                                     waiting for a resource to be realized.
+    :param realization_wait_sec: Number of seconds to wait between attempts
+                                 for a resource to be realized.
     """
 
     def __init__(self,
@@ -87,6 +103,8 @@ class NsxLibConfig(object):
                  client_cert_provider=None,
                  insecure=True,
                  ca_file=None,
+                 thumbprint=None,
+                 token_provider=None,
                  concurrent_connections=10,
                  retries=3,
                  http_timeout=10,
@@ -102,12 +120,16 @@ class NsxLibConfig(object):
                  dhcp_profile_uuid=None,
                  allow_overwrite_header=False,
                  rate_limit_retry=True,
-                 cluster_unavailable_retry=False):
+                 cluster_unavailable_retry=False,
+                 allow_passthrough=False,
+                 realization_max_attempts=50,
+                 realization_wait_sec=1.0):
 
         self.nsx_api_managers = nsx_api_managers
         self._username = username
         self._password = password
         self._ca_file = ca_file
+        self._thumbprint = thumbprint
         self.insecure = insecure
         self.concurrent_connections = concurrent_connections
         self.retries = retries
@@ -116,6 +138,7 @@ class NsxLibConfig(object):
         self.conn_idle_timeout = conn_idle_timeout
         self.http_provider = http_provider
         self.client_cert_provider = client_cert_provider
+        self.token_provider = token_provider
         self.max_attempts = max_attempts
         self.plugin_scope = plugin_scope
         self.plugin_tag = plugin_tag
@@ -125,6 +148,9 @@ class NsxLibConfig(object):
         self.allow_overwrite_header = allow_overwrite_header
         self.rate_limit_retry = rate_limit_retry
         self.cluster_unavailable_retry = cluster_unavailable_retry
+        self.allow_passthrough = allow_passthrough
+        self.realization_max_attempts = realization_max_attempts
+        self.realization_wait_sec = realization_wait_sec
 
         if dhcp_profile_uuid:
             # this is deprecated, and never used.
@@ -133,9 +159,11 @@ class NsxLibConfig(object):
                 'dhcp_profile_uuid is not used by the nsxlib, and will '
                 'be removed from its configuration in the future.')
 
-    def extend(self, keepalive_section, url_base=None):
+    def extend(self, keepalive_section, validate_connection_method=None,
+               url_base=None):
         """Called by library code to initialize application-specific data"""
         self.keepalive_section = keepalive_section
+        self.validate_connection_method = validate_connection_method
         self.url_base = url_base
 
     def _attribute_by_index(self, scalar_or_list, index):
@@ -157,3 +185,6 @@ class NsxLibConfig(object):
 
     def ca_file(self, index):
         return self._attribute_by_index(self._ca_file, index)
+
+    def thumbprint(self, index):
+        return self._attribute_by_index(self._thumbprint, index)
