@@ -74,6 +74,61 @@ class NsxLibPortMirror(utils.NsxLibApiBase):
         self.client.delete(self.get_path(mirror_session_id))
 
 
+class NsxLibBridgeEndpointProfile(utils.NsxLibApiBase):
+
+    @property
+    def uri_segment(self):
+        return 'bridge-endpoint-profiles'
+
+    @property
+    def resource_type(self):
+        return 'BridgeEndpointProfile'
+
+    def create(self, display_name, edge_cluster_id, tags,
+               edge_cluster_member_indexes=None, failover_mode=None):
+        """Create a bridge endpoint profile on the backend.
+
+        Create a bridge endpoint profile for a given edge cluster.
+        :param display_name: name of the bridge endpoint profile
+        :param edge_cluster_id: identifier of the edge cluster this profile
+                                should be associated with.
+        :param tags: tags for the newly created resource.
+        :param edge_cluster_member_indexes: iterable of integers specifying
+                                            edge cluster members where the
+                                            bridge endpoints will be created
+        :param failover_mode: failover mode for the profile. Could be either
+                              PREEMPTIVE or NON_PREEMPTIVE.
+        """
+        tags = tags or []
+        body = {'display_name': display_name,
+                'tags': tags}
+        if failover_mode:
+            body['failover_mode'] = failover_mode
+        if edge_cluster_member_indexes:
+            # Test for a list of integers
+            try:
+                member_indexes = [int(member_idx) for member_idx in
+                                  edge_cluster_member_indexes]
+                body['edge_cluster_member_indexes'] = member_indexes
+            except (TypeError, ValueError) as e:
+                LOG.Error("Invalid values for member indexes: %s", e)
+                raise exceptions.InvalidInput(
+                    operation='Create BridgeEndpointProfile',
+                    arg_val=edge_cluster_member_indexes,
+                    arg_name='edge_cluster_member_indexes')
+
+        return self.client.create(self.get_path(), body)
+
+    def delete(self, bridge_endpoint_profile_id):
+        """Delete a bridge endpoint profile on the backend.
+
+        :param bridge_endpoint_profile_id: string representing the UUID of
+                                           the bridge endpoint profile to be
+                                           deleted.
+        """
+        self.client.delete(self.get_path(bridge_endpoint_profile_id))
+
+
 class NsxLibBridgeEndpoint(utils.NsxLibApiBase):
 
     @property
@@ -84,19 +139,24 @@ class NsxLibBridgeEndpoint(utils.NsxLibApiBase):
     def resource_type(self):
         return 'BridgeEndpoint'
 
-    def create(self, device_name, seg_id, tags):
+    def create(self, device_name, vlan_transport_zone_id, vlan_id, tags):
         """Create a bridge endpoint on the backend.
 
         Create a bridge endpoint resource on a bridge cluster for the L2
         gateway network connection.
         :param device_name: device_name actually refers to the bridge cluster's
                             UUID.
-        :param seg_id: integer representing the VLAN segmentation ID.
+        :param vlan_transport_zone_id: identifier of the transport zone id
+                                       where the endpoint will be created.
+                                       Mandatory for endpoints on edge
+                                       clusters.
+        :param vlan_id: integer representing the VLAN segmentation ID.
         :param tags: nsx backend specific tags.
         """
-        body = {'bridge_cluster_id': device_name,
+        body = {'bridge_endpoint_profile_id': device_name,
+                'vlan_transport_zone_id': vlan_transport_zone_id,
                 'tags': tags,
-                'vlan': seg_id}
+                'vlan': vlan_id}
         return self.client.create(self.get_path(), body)
 
     def delete(self, bridge_endpoint_id):
@@ -291,10 +351,12 @@ class NsxLibSwitchingProfile(utils.NsxLibApiBase):
                            bpdu_filter=bpdu_filter,
                            block_non_ip_traffic=True)
 
-    def create_mac_learning_profile(self, display_name,
-                                    description, tags=None):
+    def create_mac_learning_profile(self, display_name, description,
+                                    mac_learning_enabled=True,
+                                    tags=None):
         mac_learning = {
-            'enabled': True,
+            'enabled': mac_learning_enabled,
+            'unicast_flooding_allowed': mac_learning_enabled
         }
         return self.create(SwitchingProfileTypes.MAC_LEARNING,
                            display_name=display_name,
@@ -569,7 +631,13 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
             body['display_name'] = display_name
         return self.client.create(resource, body)
 
-    def add_static_route(self, logical_router_id, dest_cidr, nexthop):
+    def change_edge_firewall_status(self, logical_router_id, action):
+        resource = 'firewall/status/logical_routers/%s?action=%s' % (
+            logical_router_id, action)
+        return self.client.create(resource)
+
+    def add_static_route(self, logical_router_id, dest_cidr, nexthop,
+                         tags=None):
         resource = ('logical-routers/%s/routing/static-routes' %
                     logical_router_id)
         body = {}
@@ -577,6 +645,8 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
             body['network'] = dest_cidr
         if nexthop:
             body['next_hops'] = [{"ip_address": nexthop}]
+        if tags is not None:
+            body['tags'] = tags
         return self.client.create(resource, body)
 
     def delete_static_route(self, logical_router_id, static_route_id):
@@ -639,10 +709,40 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
 
         return self._update_resource(resource, kwargs, retry=True)
 
-    def update_advertisement_rules(self, logical_router_id, rules):
+    def update_advertisement_rules(self, logical_router_id, rules,
+                                   name_prefix=None, force=False):
+        """Update the router advertisement rules
+
+        If name_prefix is None, replace the entire list of NSX rules with the
+        new given 'rules'.
+        Else - delete the NSX rules with this name prefix, and add 'rules' to
+        the rest.
+        """
         resource = ('logical-routers/%s/routing/advertisement/rules' %
                     logical_router_id)
-        return self._update_resource(resource, {'rules': rules}, retry=True)
+        callback = None
+
+        def update_payload_cbk(revised_payload, requested_payload):
+            # delete rules with this prefix:
+            new_rules = []
+            for rule in revised_payload['rules']:
+                if (not rule.get('display_name') or
+                    not rule['display_name'].startswith(name_prefix)):
+                    new_rules.append(rule)
+            # add new rules
+            new_rules.extend(requested_payload['rules'])
+            revised_payload['rules'] = new_rules
+            del requested_payload['rules']
+
+        if name_prefix:
+            callback = update_payload_cbk
+
+        # In case of updating advertisement rule on the logical router
+        # owned by other Principal Entities, need to force the overwrite
+        headers = {'X-Allow-Overwrite': 'true'} if force else None
+        return self._update_resource(
+            resource, {'rules': rules}, retry=True,
+            update_payload_cbk=callback, headers=headers)
 
     def get_advertisement_rules(self, logical_router_id):
         resource = ('logical-routers/%s/routing/advertisement/rules' %
@@ -657,14 +757,17 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
     def get_transportzone_id(self, logical_router_id):
         res = self.get_debug_info(logical_router_id)
         for item in res['componentInfo']:
-            if item['componentType'] == nsx_constants.ROUTER_TYPE_TIER0_DR:
+            if item['componentType'] in (nsx_constants.ROUTER_TYPE_TIER0_DR,
+                                         nsx_constants.ROUTER_TYPE_TIER1_DR):
                 if item['transportZoneId']:
                     return item['transportZoneId'][0]
         LOG.warning('OverlayTransportZone is not yet available on'
                     ' %s.' % (logical_router_id))
 
     def create(self, display_name, tags, edge_cluster_uuid=None, tier_0=False,
-               description=None, transport_zone_id=None, allocation_pool=None):
+               description=None, transport_zone_id=None,
+               allocation_pool=None, enable_standby_relocation=False,
+               failover_mode=None):
         # TODO(salv-orlando): If possible do not manage edge clusters
         # in the main plugin logic.
         router_type = (nsx_constants.ROUTER_TYPE_TIER0 if tier_0 else
@@ -679,9 +782,19 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
         if transport_zone_id:
             body['advanced_config'] = {
                 'transport_zone_id': transport_zone_id}
+        if failover_mode:
+            body['failover_mode'] = failover_mode
+        allocation_profile = {}
         if allocation_pool:
-            body['allocation_profile'] = {
-                'allocation_pool': allocation_pool}
+            allocation_profile['allocation_pool'] = allocation_pool
+        if (enable_standby_relocation and self.nsxlib and
+                self.nsxlib.feature_supported(
+                nsx_constants.FEATURE_ENABLE_STANDBY_RELOCATION)):
+            allocation_profile[
+                'enable_standby_relocation'] = enable_standby_relocation
+        if allocation_profile:
+            body['allocation_profile'] = allocation_profile
+
         return self.client.create(self.get_path(), body=body)
 
     def delete(self, lrouter_id, force=False):
@@ -697,6 +810,12 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
             if arg == 'transport_zone_id':
                 body['advanced_config'] = {
                     'transport_zone_id': kwargs['transport_zone_id']}
+            elif arg == 'enable_standby_relocation':
+                if (self.nsxlib and self.nsxlib.feature_supported(
+                        nsx_constants.FEATURE_ENABLE_STANDBY_RELOCATION)):
+                    body['allocation_profile'] = {
+                        'enable_standby_relocation':
+                            kwargs['enable_standby_relocation']}
             else:
                 body[arg] = kwargs[arg]
 
@@ -725,6 +844,35 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
             resource = self.get_path()
         return self.client.list(resource)
 
+    def get_redistribution(self, logical_router_id):
+        resource = ('logical-routers/%s/routing/redistribution' %
+                    logical_router_id)
+        return self.client.get(resource)
+
+    def get_redistribution_rules(self, logical_router_id):
+        resource = ('logical-routers/%s/routing/redistribution/rules' %
+                    logical_router_id)
+        return self.client.get(resource)
+
+    def update_redistribution_rules(self, logical_router_id, rules):
+        resource = ('logical-routers/%s/routing/redistribution/rules' %
+                    logical_router_id)
+        return self._update_resource(resource, {'rules': rules}, retry=True)
+
+    def get_bgp_config(self, logical_router_id):
+        resource = ('logical-routers/%s/routing/bgp' % logical_router_id)
+        return self.client.get(resource)
+
+    def get_route_map(self, logical_router_id, route_map_id):
+        resource = ('logical-routers/%s/routing/route-maps/%s' % (
+                    logical_router_id, route_map_id))
+        return self.client.get(resource)
+
+    def get_ip_prefix_list(self, logical_router_id, ip_prefix_list_id):
+        resource = ('logical-routers/%s/routing/ip-prefix-lists/%s' % (
+                    logical_router_id, ip_prefix_list_id))
+        return self.client.get(resource)
+
 
 class NsxLibEdgeCluster(utils.NsxLibApiBase):
 
@@ -736,6 +884,10 @@ class NsxLibEdgeCluster(utils.NsxLibApiBase):
     def resource_type(self):
         return 'EdgeCluster'
 
+    @property
+    def use_cache_for_get(self):
+        return True
+
     def get_transport_nodes(self, uuid):
         ec = self.get(uuid)
         members = []
@@ -746,10 +898,10 @@ class NsxLibEdgeCluster(utils.NsxLibApiBase):
 
 class NsxLibTransportZone(utils.NsxLibApiBase):
 
-    TRANSPORT_TYPE_VLAN = 'VLAN'
-    TRANSPORT_TYPE_OVERLAY = 'OVERLAY'
-    HOST_SWITCH_MODE_ENS = 'ENS'
-    HOST_SWITCH_MODE_STANDARD = 'STANDARD'
+    TRANSPORT_TYPE_VLAN = nsx_constants.TRANSPORT_TYPE_VLAN
+    TRANSPORT_TYPE_OVERLAY = nsx_constants.TRANSPORT_TYPE_OVERLAY
+    HOST_SWITCH_MODE_ENS = nsx_constants.HOST_SWITCH_MODE_ENS
+    HOST_SWITCH_MODE_STANDARD = nsx_constants.HOST_SWITCH_MODE_STANDARD
 
     @property
     def uri_segment(self):
@@ -787,9 +939,21 @@ class NsxLibTransportNode(utils.NsxLibApiBase):
         return True
 
     def get_transport_zones(self, uuid):
-        tz = self.get(uuid)
-        return [ep.get('transport_zone_id') for ep in
-                tz.get('transport_zone_endpoints', [])]
+        tn = self.get(uuid)
+
+        if (self.nsxlib and self.nsxlib.feature_supported(
+            nsx_constants.FEATURE_GET_TZ_FROM_SWITCH)):
+            if (not tn.get('host_switch_spec') or not
+                tn['host_switch_spec'].get('host_switches')):
+                return []
+
+            host_switches = tn.get('host_switch_spec').get('host_switches', [])
+
+            return [ep.get('transport_zone_id') for ep in
+                    host_switches[0].get('transport_zone_endpoints', [])]
+        else:
+            return [ep.get('transport_zone_id') for ep in
+                    tn.get('transport_zone_endpoints', [])]
 
 
 class NsxLibDhcpProfile(utils.NsxLibApiBase):
@@ -854,6 +1018,10 @@ class NsxLibMetadataProxy(utils.NsxLibApiBase):
     def resource_type(self):
         return 'MetadataProxy'
 
+    @property
+    def use_cache_for_get(self):
+        return True
+
     def update(self, uuid, server_url=None, secret=None, edge_cluster_id=None):
         body = {}
         # update the relevant fields
@@ -893,11 +1061,17 @@ class NsxLibIpBlockSubnet(utils.NsxLibApiBase):
     def resource_type(self):
         return 'IpBlockSubnet'
 
-    def create(self, ip_block_id, subnet_size):
+    def create(self, ip_block_id, subnet_size, allow_overwrite=False):
         """Create a IP block subnet on the backend."""
         body = {'size': subnet_size,
                 'block_id': ip_block_id}
-        return self.client.create(self.get_path(), body)
+        headers = None
+        if allow_overwrite:
+            # In case of manager to policy API resources imports,
+            # a Policy owned Manager IpBlock resource might be needed
+            # to allocate subnet using Manager APIs.
+            headers = {'X-Allow-Overwrite': 'true'}
+        return self.client.create(self.get_path(), body, headers=headers)
 
     def delete(self, subnet_id):
         """Delete a IP block subnet on the backend."""
@@ -947,3 +1121,26 @@ class NsxLibFabricVirtualInterface(utils.NsxLibApiBase):
     def get_by_owner_vm_id(self, owner_vm_id):
         url = '%s?owner_vm_id=%s' % (self.get_path(), owner_vm_id)
         return self.client.get(url)
+
+
+class NsxLibGlobalRoutingConfig(utils.NsxLibApiBase):
+
+    @property
+    def uri_segment(self):
+        return 'global-configs/RoutingGlobalConfig'
+
+    @property
+    def resource_type(self):
+        return 'RoutingGlobalConfig'
+
+    def set_l3_forwarding_mode(self, mode):
+        config = self.client.get(self.get_path())
+        if config['l3_forwarding_mode'] != mode:
+            config['l3_forwarding_mode'] = mode
+            self.client.update(self.get_path(), config)
+
+    def enable_ipv6(self):
+        return self.set_l3_forwarding_mode('IPV4_AND_IPV6')
+
+    def disable_ipv6(self):
+        return self.set_l3_forwarding_mode('IPV4_ONLY')
