@@ -16,6 +16,7 @@
 from oslo_log import log as logging
 
 from vmware_nsxlib.v3 import exceptions as nsxlib_exc
+from vmware_nsxlib.v3 import nsx_constants
 from vmware_nsxlib.v3 import utils
 
 LOG = logging.getLogger(__name__)
@@ -169,16 +170,22 @@ class ApplicationProfile(LoadBalancerBase):
             body['tags'] = tags
         if resource_type is None:
             return body
-        if resource_type == ApplicationProfileTypes.HTTP:
+        if resource_type in [ApplicationProfileTypes.HTTP,
+                             ApplicationProfileTypes.FAST_TCP,
+                             ApplicationProfileTypes.FAST_UDP]:
             body['resource_type'] = resource_type
-            extra_args = ['http_redirect_to', 'http_redirect_to_https',
-                          'ntlm', 'request_header_size', 'x_forwarded_for',
-                          'idle_timeout']
-            return utils.build_extra_args(body, extra_args, **kwargs)
-        elif (resource_type == ApplicationProfileTypes.FAST_TCP or
-              resource_type == ApplicationProfileTypes.FAST_UDP):
-            body['resource_type'] = resource_type
-            extra_args = ['ha_flow_mirroring_enabled', 'idle_timeout']
+            extra_args = ['idle_timeout']
+            if resource_type == ApplicationProfileTypes.HTTP:
+                extra_args.extend(
+                    ['http_redirect_to', 'http_redirect_to_https', 'ntlm',
+                     'request_body_size', 'request_header_size',
+                     'response_header_size', 'response_timeout',
+                     'x_forwarded_for'])
+            elif resource_type == ApplicationProfileTypes.FAST_TCP:
+                extra_args.extend(
+                    ['close_timeout', 'ha_flow_mirroring_enabled'])
+            elif resource_type == ApplicationProfileTypes.FAST_UDP:
+                extra_args.extend(['flow_mirroring_enabled'])
             return utils.build_extra_args(body, extra_args, **kwargs)
         else:
             raise nsxlib_exc.InvalidInput(
@@ -246,7 +253,7 @@ class Monitor(LoadBalancerBase):
             extra_args = ['fall_count', 'interval', 'monitor_port',
                           'request_body', 'request_method', 'request_url',
                           'request_version', 'response_body',
-                          'response_status', 'rise_count', 'timeout']
+                          'response_status_codes', 'rise_count', 'timeout']
             return utils.build_extra_args(body, extra_args, **kwargs)
         elif resource_type == MonitorTypes.HTTPS:
             body['resource_type'] = resource_type
@@ -254,8 +261,8 @@ class Monitor(LoadBalancerBase):
                           'client_certificate_id', 'fall_count', 'interval',
                           'monitor_port', 'protocols', 'request_body',
                           'request_method', 'request_url', 'request_version',
-                          'response_body', 'response_status', 'rise_count',
-                          'server_auth', 'server_auth_ca_ids',
+                          'response_body', 'response_status_codes',
+                          'rise_count', 'server_auth', 'server_auth_ca_ids',
                           'server_auth_crl_ids', 'timeout']
             return utils.build_extra_args(body, extra_args, **kwargs)
         elif resource_type == MonitorTypes.ICMP:
@@ -411,6 +418,27 @@ class VirtualServer(LoadBalancerBase):
 class Service(LoadBalancerBase):
     resource = 'loadbalancer/services'
 
+    def _build_args(self, body, display_name=None, description=None,
+                    tags=None, resource_type=None, **kwargs):
+        if display_name:
+            body['display_name'] = display_name
+        if description:
+            body['description'] = description
+        if tags:
+            body['tags'] = tags
+        if resource_type:
+            body['resource_type'] = resource_type
+
+        if ('relax_scale_validation' in kwargs and
+            not self.nsxlib.feature_supported(
+                nsx_constants.FEATURE_RELAX_SCALE_VALIDATION)):
+            kwargs.pop('relax_scale_validation')
+            LOG.warning("Ignoring relax_scale_validation for new "
+                        "lb service %s: this feature is not supported.",
+                        display_name)
+        body.update(kwargs)
+        return body
+
     def update_service_with_virtual_servers(self, service_id,
                                             virtual_server_ids):
         # Using internal method so we can access max_attempts in the decorator
@@ -424,7 +452,8 @@ class Service(LoadBalancerBase):
             return self.client.update(object_url, body)
         return do_update()
 
-    def update_service_with_attachment(self, service_id, logical_router_id):
+    def update_service_with_attachment(self, service_id, logical_router_id,
+                                       tags=None):
         # Using internal method so we can access max_attempts in the decorator
         @utils.retry_upon_exception(
             nsxlib_exc.StaleRevision,
@@ -434,6 +463,8 @@ class Service(LoadBalancerBase):
             body = self.client.get(object_url)
             body['attachment'] = {'target_id': logical_router_id,
                                   'target_type': 'LogicalRouter'}
+            if tags is not None:
+                body['tags'] = tags
             return self.client.update(object_url, body)
         return do_update()
 
@@ -472,8 +503,8 @@ class Service(LoadBalancerBase):
 class LoadBalancer(object):
     """This is the class that have all load balancer resource clients"""
 
-    def __init__(self, client, nsxlib_config=None):
-        self.service = Service(client, nsxlib_config)
+    def __init__(self, client, nsxlib_config=None, nsxlib=None):
+        self.service = Service(client, nsxlib_config, nsxlib)
         self.virtual_server = VirtualServer(client, nsxlib_config)
         self.pool = Pool(client, nsxlib_config)
         self.monitor = Monitor(client, nsxlib_config)
