@@ -82,7 +82,8 @@ class LogicalPort(utils.NsxLibApiBase):
             address_bindings=None,
             switch_profile_ids=None,
             attachment=None,
-            description=None):
+            description=None,
+            extra_configs=None):
         tags = tags or []
         switch_profile_ids = switch_profile_ids or []
         body = {}
@@ -127,6 +128,9 @@ class LogicalPort(utils.NsxLibApiBase):
         if description is not None:
             body['description'] = description
 
+        if extra_configs:
+            body['extra_configs'] = extra_configs
+
         return body
 
     def _prepare_attachment(self, attachment_type, vif_uuid,
@@ -159,7 +163,8 @@ class LogicalPort(utils.NsxLibApiBase):
                parent_vif_id=None, traffic_tag=None,
                switch_profile_ids=None, vif_type=None, app_id=None,
                allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE,
-               description=None, tn_uuid=None):
+               description=None, tn_uuid=None,
+               extra_configs=None):
         tags = tags or []
         body = {'logical_switch_id': lswitch_id}
         # NOTE(arosen): If parent_vif_id is specified we need to use
@@ -174,7 +179,8 @@ class LogicalPort(utils.NsxLibApiBase):
             address_bindings=address_bindings,
             switch_profile_ids=switch_profile_ids,
             attachment=attachment,
-            description=description))
+            description=description,
+            extra_configs=extra_configs))
         return self.client.create(self.get_path(), body=body)
 
     def delete(self, lport_id):
@@ -183,12 +189,19 @@ class LogicalPort(utils.NsxLibApiBase):
     def update(self, lport_id, vif_uuid,
                name=None, admin_state=None,
                address_bindings=None, switch_profile_ids=None,
-               tags_update=None,
+               tags_update=None, tags=None,
                attachment_type=nsx_constants.ATTACHMENT_VIF,
                parent_vif_id=None, traffic_tag=None,
                vif_type=None, app_id=None,
                allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE,
-               description=None, tn_uuid=None):
+               description=None, tn_uuid=None,
+               extra_configs=None, force=False):
+        # Do not allow tags & tags_update at the same call
+        if tags_update and tags:
+            raise exceptions.ManagerError(
+                details=_("Can't support updating logical port %s both with "
+                          "tags and tags_update attributes") % lport_id)
+
         attachment = self._prepare_attachment(attachment_type, vif_uuid,
                                               allocate_addresses, vif_type,
                                               parent_vif_id, traffic_tag,
@@ -202,10 +215,16 @@ class LogicalPort(utils.NsxLibApiBase):
             address_bindings=address_bindings,
             switch_profile_ids=switch_profile_ids,
             attachment=attachment,
-            description=description))
+            description=description,
+            extra_configs=extra_configs,
+            tags=tags))
+
+        headers = None
+        if force:
+            headers = {'X-Allow-Overwrite': 'true'}
 
         return self._update_resource(
-            self.get_path(lport_id), lport, retry=True)
+            self.get_path(lport_id), lport, headers=headers, retry=True)
 
     def get_by_attachment(self, attachment_type, attachment_id):
         """Return all logical port matching the attachment type and Id"""
@@ -213,6 +232,11 @@ class LogicalPort(utils.NsxLibApiBase):
                       (attachment_type, attachment_id))
         return self.client.get(self.get_path(url_suffix))
 
+    def get_by_logical_switch(self, logical_switch_id):
+        """Return all logical port of a logical switch"""
+        url_suffix = '?logical_switch_id=%s' % logical_switch_id
+        return self.client.get(self.get_path(url_suffix))
+
 
 class LogicalRouter(core_resources.NsxLibLogicalRouter):
     # TODO(asarfaty): keeping this for backwards compatibility.
@@ -352,13 +376,29 @@ class LogicalRouterPort(utils.NsxLibApiBase):
             if port['resource_type'] == nsx_constants.LROUTERPORT_UPLINK:
                 return port
 
-    def get_tier0_uplink_ips(self, logical_router_id):
+    def get_tier0_uplink_subnets(self, logical_router_id):
         port = self.get_tier0_uplink_port(logical_router_id)
-        ips = []
         if port:
-            for subnet in port.get('subnets', []):
-                for ip_address in subnet.get('ip_addresses'):
-                    ips.append(ip_address)
+            return port.get('subnets', [])
+        return []
+
+    def get_tier0_uplink_cidrs(self, logical_router_id):
+        # return a list of tier0 uplink ip/prefix addresses
+        subnets = self.get_tier0_uplink_subnets(logical_router_id)
+        cidrs = []
+        for subnet in subnets:
+            for ip_address in subnet.get('ip_addresses'):
+                cidrs.append('%s/%s' % (ip_address,
+                                        subnet.get('prefix_length')))
+        return cidrs
+
+    def get_tier0_uplink_ips(self, logical_router_id):
+        # return a list of tier0 uplink ip addresses
+        subnets = self.get_tier0_uplink_subnets(logical_router_id)
+        ips = []
+        for subnet in subnets:
+            for ip_address in subnet.get('ip_addresses'):
+                ips.append(ip_address)
         return ips
 
 
@@ -387,49 +427,7 @@ class DhcpProfile(core_resources.NsxLibDhcpProfile):
 class LogicalDhcpServer(utils.NsxLibApiBase):
 
     def get_dhcp_opt_code(self, name):
-        _supported_options = {
-            'subnet-mask': 1,
-            'time-offset': 2,
-            'router': 3,
-            'dns-name': 6,
-            'host-name': 12,
-            'boot-file-size': 13,
-            'domain-name': 15,
-            'ip-forwarding': 19,
-            'interface-mtu': 26,
-            'broadcast-address': 28,
-            'arp-cache-timeout': 35,
-            'nis-domain': 40,
-            'nis-servers': 41,
-            'ntp-servers': 42,
-            'netbios-name-servers': 44,
-            'netbios-dd-server': 45,
-            'netbios-node-type': 46,
-            'netbios-scope': 47,
-            'dhcp-renewal-time': 58,
-            'dhcp-rebinding-time': 59,
-            'class-id': 60,
-            'dhcp-client-identifier': 61,
-            'nisplus-domain': 64,
-            'nisplus-servers': 65,
-            'tftp-server': 66,
-            'tftp-server-name': 66,
-            'bootfile-name': 67,
-            'system-architecture': 93,
-            'interface-id': 94,
-            'machine-id': 97,
-            'name-search': 117,
-            'subnet-selection': 118,
-            'domain-search': 119,
-            'classless-static-route': 121,
-            'tftp-server-address': 150,
-            'server-ip-address': 150,
-            'etherboot': 175,
-            'config-file': 209,
-            'path-prefix': 210,
-            'reboot-time': 211,
-        }
-        return _supported_options.get(name)
+        return utils.get_dhcp_opt_code(name)
 
     @property
     def uri_segment(self):
@@ -726,3 +724,108 @@ class NsxlibClusterNodesConfig(utils.NsxLibApiBase):
                         manager_ips.append(list_addr['ip_address'])
 
         return manager_ips
+
+
+class NsxlibHostSwitchProfiles(utils.NsxLibApiBase):
+    @property
+    def uri_segment(self):
+        return 'host-switch-profiles'
+
+    @property
+    def resource_type(self):
+        return 'UplinkHostSwitchProfile'
+
+
+class Inventory(utils.NsxLibApiBase):
+    """REST APIs to support inventory service."""
+
+    RESOURCES_PATH = {"ContainerCluster": "container-clusters",
+                      "ContainerProject": "container-projects",
+                      "ContainerApplication": "container-applications",
+                      "ContainerApplicationInstance":
+                      "container-application-instances",
+                      "ContainerClusterNode": "container-cluster-nodes",
+                      "ContainerNetworkPolicy": "container-network-policies",
+                      "ContainerIngressPolicy": "container-ingress-policies"}
+    SEGMENT_URI = 'fabric'
+    SEGMENT_URI_FOR_UPDATE = 'inventory/container'
+
+    @property
+    def uri_segment(self):
+        # only serves for get, list and delete. For batch update,
+        # another base url is used. The reason is update API is internal.
+        return self.SEGMENT_URI
+
+    @property
+    def resource_type(self):
+        return 'Inventory'
+
+    def update(self, cluster_id, updates):
+        """This method supports multiple updates in a batching way."""
+        items = []
+        for update_type, update_object in updates:
+            item = {}
+            item["object_update_type"] = update_type
+            item["container_object"] = update_object
+            items.append(item)
+        body = {"container_inventory_objects": items}
+        request_url = "%s/%s?action=updates" % (
+            self.SEGMENT_URI_FOR_UPDATE, cluster_id)
+        return self.client.url_post(request_url, body)
+
+    def get(self, resource_type, resource_id):
+        if not resource_type:
+            msg = "null resource type is not supported"
+            raise exceptions.ResourceNotFound(details=msg)
+        request_url = "%s/%s" % (
+            self.get_path(self._get_path_for_resource(resource_type)),
+            resource_id)
+        return self.client.url_get(request_url)
+
+    def list(self, cluster_id, resource_type):
+        if not resource_type:
+            msg = "null resource type is not supported"
+            raise exceptions.ResourceNotFound(details=msg)
+        request_url = "%s?container_cluster_id=%s" % (
+            self.get_path(self._get_path_for_resource(resource_type)),
+            cluster_id)
+        return self.client.url_get(request_url)
+
+    def delete(self, resource_type, resource_id):
+        if not resource_type:
+            msg = "null resource type is not supported"
+            raise exceptions.ResourceNotFound(details=msg)
+        request_url = "%s/%s" % (
+            self.get_path(self._get_path_for_resource(resource_type)),
+            resource_id)
+        return self.client.url_delete(request_url)
+
+    def create(self, resource_type, resource):
+        if not resource_type:
+            msg = "null resource type is not supported"
+            raise exceptions.ResourceNotFound(details=msg)
+        request_url = self.get_path(
+            self._get_path_for_resource(resource_type))
+        return self.client.url_post(request_url, resource)
+
+    def _get_path_for_resource(self, resource_type):
+        path = self.RESOURCES_PATH.get(resource_type)
+        if not path:
+            msg = "backend resource %s is not supported" % resource_type
+            raise exceptions.ResourceNotFound(details=msg)
+        return path
+
+
+class SystemHealth(utils.NsxLibApiBase):
+    @property
+    def uri_segment(self):
+        return 'systemhealth'
+
+    @property
+    def resource_type(self):
+        return 'SystemHealth'
+
+    def create_ncp_status(self, cluster_id, status):
+        url = '/container-cluster/ncp/status'
+        body = {'cluster_id': cluster_id, 'status': status}
+        return self.client.create(self.get_path(url), body=body)
