@@ -56,7 +56,7 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
         name = self.get_name(security_group)
         description = security_group['description']
         logging = (log_sg_allowed_traffic or
-                   security_group[consts.LOGGING])
+                   security_group.get(consts.LOGGING, False))
         rules = self.firewall_section._process_rules_logging_for_update(
             section_id, logging)
         self.update(nsgroup_id, name, description)
@@ -469,30 +469,22 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
         return rule_dict
 
     def add_rule(self, rule, section_id, operation=consts.FW_INSERT_BOTTOM):
-        @utils.retry_upon_exception(exceptions.StaleRevision,
-                                    max_attempts=self.client.max_attempts)
-        def do_add_rule():
-            resource = '%s/rules' % self.get_path(section_id)
-            params = '?operation=%s' % operation
-            if (version.LooseVersion(self.nsxlib.get_version()) >=
-                version.LooseVersion(consts.NSX_VERSION_2_4_0)):
-                rule['_revision'] = self.get(section_id)['_revision']
-            return self._create_with_retry(resource + params, rule)
-        return do_add_rule()
+        resource = '%s/rules' % self.get_path(section_id)
+        params = '?operation=%s' % operation
+        if (version.LooseVersion(self.nsxlib.get_version()) >=
+            version.LooseVersion(consts.NSX_VERSION_2_4_0)):
+            rule['_revision'] = self.get(section_id)['_revision']
+        return self._create_with_retry(resource + params, rule)
 
     def add_rules(self, rules, section_id, operation=consts.FW_INSERT_BOTTOM):
-        @utils.retry_upon_exception(exceptions.StaleRevision,
-                                    max_attempts=self.client.max_attempts)
-        def do_add_rules():
-            resource = '%s/rules' % self.get_path(section_id)
-            params = '?action=create_multiple&operation=%s' % operation
-            if (version.LooseVersion(self.nsxlib.get_version()) >=
-                version.LooseVersion(consts.NSX_VERSION_2_4_0)):
-                rev_id = self.get(section_id)['_revision']
-                for rule in rules:
-                    rule['_revision'] = rev_id
-            return self._create_with_retry(resource + params, {'rules': rules})
-        return do_add_rules()
+        resource = '%s/rules' % self.get_path(section_id)
+        params = '?action=create_multiple&operation=%s' % operation
+        if (version.LooseVersion(self.nsxlib.get_version()) >=
+            version.LooseVersion(consts.NSX_VERSION_2_4_0)):
+            rev_id = self.get(section_id)['_revision']
+            for rule in rules:
+                rule['_revision'] = rev_id
+        return self._create_with_retry(resource + params, {'rules': rules})
 
     def delete_rule(self, section_id, rule_id):
         resource = '%s/rules/%s' % (section_id, rule_id)
@@ -656,7 +648,16 @@ class NsxLibIPSet(utils.NsxLibApiBase):
         return self.client.create(self.get_path(), body)
 
     def update(self, ip_set_id, display_name=None, description=None,
-               ip_addresses=None, tags_update=None):
+               ip_addresses=None, tags_update=None, update_payload_cbk=None):
+        # The update_payload_cbk function takes two arguments.
+        # The first one is the result from the internal GET request.
+        # The second one is a dict of user-provided attributes,
+        # which can be changed inside the callback function and
+        # used as the new payload for the following PUT request.
+        # For example, users want to combine the new ip_addresses
+        # passed to update() with the original ip_addresses retrieved
+        # from the internal GET request instead of overriding the
+        # original ip_addresses.
         ip_set = {}
         if tags_update:
             ip_set['tags_update'] = tags_update
@@ -667,7 +668,8 @@ class NsxLibIPSet(utils.NsxLibApiBase):
         if ip_addresses is not None:
             ip_set['ip_addresses'] = ip_addresses
         return self._update_resource(self.get_path(ip_set_id),
-                                     ip_set, retry=True)
+                                     ip_set, retry=True,
+                                     update_payload_cbk=update_payload_cbk)
 
     def read(self, ip_set_id):
         return self.client.get('ip-sets/%s' % ip_set_id)
