@@ -12,6 +12,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+
+import six
+
 from vmware_nsxlib.v3 import exceptions as nsxlib_exc
 from vmware_nsxlib.v3 import utils
 
@@ -26,8 +29,8 @@ USER_GROUP_TYPES = [
 
 class NsxLibTrustManagement(utils.NsxLibApiBase):
 
-    def create_cert(self, cert_pem, private_key=None, passphrase=None,
-                    tags=None):
+    def create_cert_list(self, cert_pem, private_key=None, passphrase=None,
+                         tags=None):
         resource = CERT_SECTION + '?action=import'
         body = {'pem_encoded': cert_pem}
         if private_key:
@@ -37,7 +40,14 @@ class NsxLibTrustManagement(utils.NsxLibApiBase):
             body.update({'passphrase': passphrase})
         if tags:
             body.update({'tags': tags})
-        results = self.client.create(resource, body)['results']
+        return self.client.create(resource, body)['results']
+
+    def create_cert(self, cert_pem, private_key=None, passphrase=None,
+                    tags=None):
+        results = self.create_cert_list(cert_pem, private_key, passphrase,
+                                        tags)
+        # note: the assumption of only one result is wrong. It returns the
+        # chained certs
         if len(results) > 0:
             # should be only one result
             return results[0]['id']
@@ -82,11 +92,13 @@ class NsxLibTrustManagement(utils.NsxLibApiBase):
         self.client.delete(resource)
 
     def find_cert_and_identity(self, name, cert_pem):
-        nsx_style_pem = cert_pem
         certs = self.get_certs()
 
+        if not isinstance(cert_pem, six.text_type):
+            cert_pem = cert_pem.decode('ascii')
+
         cert_ids = [cert['id'] for cert in certs
-                    if cert['pem_encoded'] == nsx_style_pem.decode('ascii')]
+                    if cert['pem_encoded'] == cert_pem]
         if not cert_ids:
             raise nsxlib_exc.ResourceNotFound(
                 manager=self.client.nsx_api_managers,
