@@ -33,10 +33,10 @@ LOG = log.getLogger(__name__)
 TagLimits = collections.namedtuple('TagLimits',
                                    ['scope_length', 'tag_length', 'max_tags'])
 
-# The tag limits may change in the NSX. We set the default values to be those
-# in NSX 2.0. If the NSX returns different values we update these globals.
-MAX_RESOURCE_TYPE_LEN = 20
-MAX_TAG_LEN = 40
+# These defaults reflect latest tag & scope limits on the backend. As of 2.5,
+# backend no longer returns tag limit via API.
+MAX_RESOURCE_TYPE_LEN = 128
+MAX_TAG_LEN = 256
 MAX_TAGS = 15
 MAX_NSGROUPS_CRITERIA_TAGS = 10
 
@@ -62,6 +62,18 @@ def set_inject_headers_callback(callback):
     INJECT_HEADERS_CALLBACK = callback
 
 
+def censor_headers(headers):
+    censored_headers = ['authorization']
+    result = {}
+    for name, value in headers.items():
+        if name.lower() in censored_headers:
+            result[name] = '--- CENSORED ---'
+        else:
+            result[name] = value
+
+    return result
+
+
 def _update_resource_length(length):
     global MAX_RESOURCE_TYPE_LEN
     MAX_RESOURCE_TYPE_LEN = length
@@ -128,12 +140,14 @@ def update_v3_tags(current_tags, tags_update):
     return tags
 
 
-def _log_before_retry(func, trial_number):
+def _log_before_retry(retry_state):
     """Before call strategy that logs to some logger the attempt."""
-    if trial_number > 1:
+    if retry_state.attempt_number > 1:
         LOG.warning("Retrying call to '%(func)s' for the %(num)s time",
-                    {'func': tenacity_utils.get_callback_name(func),
-                     'num': tenacity_utils.to_ordinal(trial_number)})
+                    {'func': tenacity_utils.get_callback_name(
+                        retry_state.fn),
+                     'num': tenacity_utils.to_ordinal(
+                        retry_state.attempt_number)})
 
 
 def _get_args_from_frame(frames, frame_num):
@@ -145,7 +159,7 @@ def _get_args_from_frame(frames, frame_num):
         return formated_args
 
 
-def _log_after_retry(func, trial_number, trial_time_taken):
+def _log_after_retry(retry_state):
     """After call strategy that logs to some logger the finished attempt."""
     # Using inspect to get arguments of the relevant call
     frames = inspect.trace()
@@ -158,9 +172,9 @@ def _log_after_retry(func, trial_number, trial_time_taken):
 
     LOG.warning("Finished retry of %(func)s for the %(num)s time after "
                 "%(time)0.3f(s) with args: %(args)s",
-                {'func': tenacity_utils.get_callback_name(func),
-                 'num': tenacity_utils.to_ordinal(trial_number),
-                 'time': trial_time_taken,
+                {'func': tenacity_utils.get_callback_name(retry_state.fn),
+                 'num': tenacity_utils.to_ordinal(retry_state.attempt_number),
+                 'time': retry_state.seconds_since_start,
                  'args': formated_args})
 
 
@@ -184,11 +198,17 @@ def retry_random_upon_exception(exc, delay=0.5, max_delay=5,
                           before=_log_before_retry, after=_log_after_retry)
 
 
-def retry_upon_none_result(max_attempts, delay=0.5, max_delay=2):
+def retry_upon_none_result(max_attempts, delay=0.5, max_delay=10,
+                           random=False):
+    if random:
+        wait_func = tenacity.wait_random_exponential(
+            multiplier=delay, max=max_delay)
+    else:
+        wait_func = tenacity.wait_exponential(
+            multiplier=delay, max=max_delay)
     return tenacity.retry(reraise=True,
                           retry=tenacity.retry_if_result(lambda x: x is None),
-                          wait=tenacity.wait_exponential(
-                              multiplier=delay, max=max_delay),
+                          wait=wait_func,
                           stop=tenacity.stop_after_attempt(max_attempts),
                           before=_log_before_retry, after=_log_after_retry)
 
@@ -254,7 +274,7 @@ def escape_tag_data(data):
     # ElasticSearch query_string requires slashes and dashes to
     # be escaped. We assume no other reserved characters will be
     # used in tag scopes or values
-    return data.replace('/', '\\/').replace('-', '\\-')
+    return data.replace('/', '\\/').replace('-', '\\-').replace(':', '\\:')
 
 
 def escape_display_name(display_name):
@@ -494,6 +514,20 @@ class NsxLibApiBase(object):
                 {'scope': "os-api-version",
                  'tag': self.nsxlib_config.plugin_ver}]
 
+    def build_v3_api_version_project_tag(self, project_name, project_id=None):
+        if not project_name:
+            project_name = self.nsxlib_config.plugin_tag
+        tags = [{'scope': self.nsxlib_config.plugin_scope,
+                 'tag': self.nsxlib_config.plugin_tag},
+                {'scope': "os-api-version",
+                 'tag': self.nsxlib_config.plugin_ver},
+                {'scope': 'os-project-name',
+                 'tag': project_name[:MAX_TAG_LEN]}]
+        if project_id:
+            tags.append({'scope': 'os-project-id',
+                         'tag': project_id[:MAX_TAG_LEN]})
+        return tags
+
     def is_internal_resource(self, nsx_resource):
         """Indicates whether the passed nsx-resource is internal
 
@@ -584,3 +618,49 @@ def get_l4_protocol_name(protocol_number):
         return nsx_constants.ICMPV4
     else:
         return protocol_number
+
+
+def get_dhcp_opt_code(name):
+    _supported_options = {
+        'subnet-mask': 1,
+        'time-offset': 2,
+        'router': 3,
+        'dns-name': 6,
+        'host-name': 12,
+        'boot-file-size': 13,
+        'domain-name': 15,
+        'ip-forwarding': 19,
+        'interface-mtu': 26,
+        'broadcast-address': 28,
+        'arp-cache-timeout': 35,
+        'nis-domain': 40,
+        'nis-servers': 41,
+        'ntp-servers': 42,
+        'netbios-name-servers': 44,
+        'netbios-dd-server': 45,
+        'netbios-node-type': 46,
+        'netbios-scope': 47,
+        'dhcp-renewal-time': 58,
+        'dhcp-rebinding-time': 59,
+        'class-id': 60,
+        'dhcp-client-identifier': 61,
+        'nisplus-domain': 64,
+        'nisplus-servers': 65,
+        'tftp-server': 66,
+        'tftp-server-name': 66,
+        'bootfile-name': 67,
+        'system-architecture': 93,
+        'interface-id': 94,
+        'machine-id': 97,
+        'name-search': 117,
+        'subnet-selection': 118,
+        'domain-search': 119,
+        'classless-static-route': 121,
+        'tftp-server-address': 150,
+        'server-ip-address': 150,
+        'etherboot': 175,
+        'config-file': 209,
+        'path-prefix': 210,
+        'reboot-time': 211,
+    }
+    return _supported_options.get(name)
